@@ -1,48 +1,57 @@
-'''
-Useful symmetry functions.
-'''
-
-import qmpy.utils._spglib as spg
-from qmpy.data import elements
 import fractions as frac
 import numpy as np
+import logging
+
+import pyspglib._spglib as spg
+
+import qmpy
+import qmpy.data as data
+from qmpy.utils import *
+
+logger = logging.getLogger(__name__)
+
+if not qmpy.FOUND_SPGLIB:
+    logger.critical('Must install spglib to be able to do symmetry analysis')
 
 ## spglib functions | http://spglib.sourceforge.net/
 
-zdict = dict( (e['z'], e['symbol']) for e in elements.values() )
-
 def find_structure_symmetry(structure, method='spglib',
         symprec=1e-5, angle_tolerance=-1.0):
     """
     Return the rotatiosn and translations which are possessed by the structure.
     
-    Example:
-    >>> from qmpy.io import read
-    >>> from qmpy.analysis.symmetry import find_structure_symmetry
-    >>> structure = read('POSCAR')
-    >>> find_structure_symmetry(structure)
+    Examples::
+
+        >>> from qmpy.io import read
+        >>> from qmpy.analysis.symmetry import find_structure_symmetry
+        >>> structure = read('POSCAR')
+        >>> find_structure_symmetry(structure)
     
     """
     # Get number of symmetry operations and allocate symmetry operations
     multi = 48 * len(structure)
     rotation = np.zeros((multi, 3, 3), dtype='intc')
     translation = np.zeros((multi, 3))
+
+    cell = structure.cell.T.copy()
+    coords = structure.site_coords.copy()
+    numbers = np.array(structure.site_ids, dtype='intc')
   
     # Get symmetry operations
     magmoms = structure.magmoms
     if not any(magmoms):
         num_sym = spg.symmetry(rotation,
                                translation,
-                               structure.cell.T.copy(),
-                               structure.scaled_coords.copy(),
-                               structure.atomic_numbers,
+                               cell,
+                               coords,
+                               numbers,
                                symprec,
                                angle_tolerance)
     else:
         num_sym = spg.symmetry_with_collinear_spin(rotation,
                                                    translation,
-                                                   lattice,
-                                                   positions,
+                                                   cell,
+                                                   coords,
                                                    numbers,
                                                    magmoms,
                                                    symprec,
@@ -69,11 +78,12 @@ def get_symmetry_dataset(structure, symprec=1e-3, angle_tolerance=-1.0):
         wyckoffs:
           Wyckoff letters
 
-    Example:
-    >>> from qmpy.io import read
-    >>> from qmpy.analysis.symmetry import get_symmetry_dataset
-    >>> structure = read('POSCAR')
-    >>> get_symmetry_dataset(structure)
+    Examples::
+
+        >>> from qmpy.io import read
+        >>> from qmpy.analysis.symmetry import get_symmetry_dataset
+        >>> structure = read('POSCAR')
+        >>> get_symmetry_dataset(structure)
 
     """
     keys = ('number',
@@ -86,16 +96,17 @@ def get_symmetry_dataset(structure, symprec=1e-3, angle_tolerance=-1.0):
             'wyckoffs',
             'equivalent_atoms')
 
-    scaled = structure.site_coords.copy()
-    labels = structure.site_labels.copy()
-    ldict = dict( (k, i+1) for i, k in enumerate(list(set(labels))))
-    numbers = np.array([ ldict[k] for k in labels ], dtype='intc')
+    cell = structure.cell.T.copy()
+    coords = np.array(structure.site_coords)
+    comps = structure.site_compositions
+    numbers = [ comps.index(c) for c in comps ]
+    numbers = np.array(numbers, dtype='intc')
 
     dataset = {}
     for key, data in zip(keys, 
             spg.dataset(
-                structure.cell.T.copy(),
-                scaled,
+                cell,
+                coords,
                 numbers,
                 symprec,
                 angle_tolerance)):
@@ -110,7 +121,6 @@ def get_symmetry_dataset(structure, symprec=1e-3, angle_tolerance=-1.0):
     letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
     dataset['wyckoffs'] = [letters[x] for x in dataset['wyckoffs']]
     dataset['equivalent_atoms'] = np.array(dataset['equivalent_atoms'])
-
     return dataset
 
 def get_spacegroup(structure, symprec=1e-5, angle_tolerance=-1.0):
@@ -118,10 +128,15 @@ def get_spacegroup(structure, symprec=1e-5, angle_tolerance=-1.0):
     Return space group in international table symbol and number
     as a string.
     """
+    cell = structure.cell.T.copy(),
+    scaled = structure.site_coords.copy()
+    comps = structure.site_compositions
+    numbers = [ comps.index(c) for c in comps ]
+    numbers = np.array(numbers, dtype='intc')
     # Atomic positions have to be specified by scaled positions for spglib.
-    return int(spg.spacegroup(structure.cell.T.copy(),
-                          structure.scaled_coords.copy(),
-                          structure.atomic_numbers,
+    return int(spg.spacegroup(cell,
+                          coords,
+                          numbers,
                           symprec,
                           angle_tolerance).strip(' ()'))
 
@@ -138,111 +153,64 @@ def refine_cell(structure, symprec=1e-5, angle_tolerance=-1.0):
     Return refined cell
     """
     # Atomic positions have to be specified by scaled positions for spglib.
-    num_atom = len(structure)
-    lattice = structure.cell.T.copy()
-    positions = np.zeros((num_atom * 4, 3), dtype='double')
-    positions[:num_atom] = structure.scaled_coords.copy()
+    num_atom = len(structure.sites)
+    cell = structure.cell.T.copy()
+    coords = np.zeros((num_atom * 4, 3), dtype='double')
+    coords[:num_atom] = structure.site_coords.copy()
+    comps = structure.site_compositions
+    numbers = [ comps.index(c) for c in comps ]
+    numbers = np.array(numbers*4, dtype='intc')
 
-    numbers = np.zeros(num_atom * 4, dtype='intc')
-    numbers[:num_atom] = structure.atomic_numbers.copy()
-    num_atom_bravais = spg.refine_cell(lattice,
-                                       positions,
+    num_atom_bravais = spg.refine_cell(cell,
+                                       coords,
                                        numbers,
                                        num_atom,
                                        symprec,
                                        angle_tolerance)
 
-    positions %= 1.0
-    positions %= 1.0
-
+    coords = wrap(coords)
+    comps = [ comps[i] for i in numbers ]
     if num_atom_bravais > 0:
-        structure.cell = lattice.T
-        structure.set_natoms(num_atom_bravais)
-        for a, z, c in zip(structure.atoms,
-                           numbers[:num_atom_bravais], 
-                           positions[:num_atom_bravais]):
-            a.element_id = zdict[z]
-            a.coord = c
-            a.direct = True
+        structure.cell = cell.T
+        structure.set_nsites(num_atom_bravais)
+        structure.site_coords = coords[:num_atom_bravais]
+        structure.site_compositions = comps[:num_atom_bravais]
         return structure
     else:
         return structure
 
 
-def find_primitive(structure, symprec=1e-5, angle_tolerance=-1.0):
+def find_primitive(structure, symprec=1e-4, angle_tolerance=-1.0):
     """
     A primitive cell in the input cell is searched and returned
     as an object of Atoms class.
     If no primitive cell is found, (None, None, None) is returned.
     """
-    lattice = structure.cell.T.copy()
-    positions = structure.scaled_coords.copy()
-    numbers = structure.atomic_numbers.copy()
+    cell = structure.cell.T.copy()
+    coords = np.array(structure.site_coords.copy(), dtype='double')
+    comps = structure.site_compositions
+    numbers = [ comps.index(c) for c in comps ]
+    numbers = np.array(numbers*4, dtype='intc')
 
-    num_atom_prim = spg.primitive(lattice,
-                                  positions,
+    num_atom_prim = spg.primitive(cell,
+                                  coords,
                                   numbers,
                                   symprec,
                                   angle_tolerance)
-    positions %= 1.0
-    positions %= 1.0
+
+    coords = wrap(coords)
+    comps = [ comps[i] for i in numbers ]
     if num_atom_prim > 0:
-        structure.cell = lattice.T
-        structure.set_natoms(num_atom_prim)
-        for a, z, c in zip(structure.atoms,
-                           numbers[:num_atom_prim], 
-                           positions[:num_atom_prim]):
-            a.element_id = zdict[z]
-            a.coord = c
-            a.direct = True
+        structure.cell = cell.T
+        structure.set_nsites(num_atom_prim)
+        structure.site_coords = coords[:num_atom_prim]
+        structure.site_compositions = comps[:num_atom_prim]
         return structure
     else:
         return structure
 
-def equivalent_reflections(hkl, rotations):
-    """Return all equivalent reflections to the list of Miller indices
-    in hkl.
-
-    Example:
-    >>> equivalent_reflections([[0, 0, 2]], rotations)
-    array([[ 0,  0, -2],
-           [ 0, -2,  0],
-           [-2,  0,  0],
-           [ 2,  0,  0],
-           [ 0,  2,  0],
-           [ 0,  0,  2]])
-    """
-    hkl = np.array(hkl, dtype='intc', ndmin=2)
-    n, nrot = len(hkl), len(rotations)
-    R = rotations.transpose(0, 2, 1).reshape((3*nrot, 3)).T
-    refl = np.dot(hkl, R).reshape((n*nrot, 3))
-    ind = np.lexsort(refl.T)
-    refl = refl[ind]
-    diff = np.diff(refl, axis=0)
-    mask = np.any(diff, axis=1)
-    return np.vstack((refl[mask], refl[-1,:]))
-
-def normalised_reflections(hkl, rotations):
-    """Returns an array of same size as *hkl*, containing the
-    corresponding symmetry-equivalent reflections of lowest
-    indices.
-
-    Example:
-    >>> sg.symmetry_normalised_reflections([[2, 0, 0], [0, 2, 0]])
-    array([[ 0,  0, -2],
-           [ 0,  0, -2]])
-    """
-    hkl = np.array(hkl, dtype='intc', ndmin=2)
-    normalised = np.empty(hkl.shape, int)
-    R = rotations.transpose(0, 2, 1)
-    for i, g in enumerate(hkl):
-        gsym = np.dot(R, g)
-        j = np.lexsort(gsym.T)[0]
-        normalised[i,:] = gsym[j]
-    return normalised
-
 def parse_sitesym(sitesym, sep=','):
-    rot = np.zeros((3, 3), dtype='intc')
+    rot = np.zeros((3, 3))
     trans = np.zeros(3)
     for i, s in enumerate (sitesym.split(sep)):
         s = s.lower().strip()
