@@ -10,6 +10,7 @@ logger = logging.getLogger(__name__)
 
 def fit(name, calculations=None, experiments=None, fit_for=[]):
     f = qmpy.Fit.get(name)
+    f.save()
 
     f.elements = fit_for
     data = []
@@ -110,7 +111,8 @@ def fit(name, calculations=None, experiments=None, fit_for=[]):
     for elt, mu in base_mus.items():
         if elt not in element_mus:
             element_mus[qmpy.Element.get(elt)] = float(mu)
-        elif abs(element_mus[elt]) > 100:
+
+        if abs(element_mus.get(qmpy.Element.get(elt), 0.0)) > 100:
             print elt, element_mus[elt], mu
             element_mus[qmpy.Element.get(elt)] = float(mu)
 
@@ -120,6 +122,8 @@ def fit(name, calculations=None, experiments=None, fit_for=[]):
     new_e_mus = {}
     for elt, val in element_mus.items():
         print elt, val
+        if val > 100:
+            val = base_mus[str(elt)]
         mu = qmpy.ReferenceEnergy(element_id=elt, value=val)
         f.reference_energy_set.add(mu)
         new_e_mus[str(elt)] = val
