@@ -73,8 +73,6 @@ class PhaseData(object):
 
         if not phase.name in self.phase_dict:
             self.phase_dict[phase.name] = phase
-        elif abs(self.phase_dict[phase.name].energy - phase.energy) < 1e-3:
-            return
         else:
             if phase.energy < self.phase_dict[phase.name].energy:
                 self.phase_dict[phase.name] = phase
@@ -143,14 +141,18 @@ class PhaseData(object):
         from qmpy.materials.element import Element
         logger.debug('Loading Phases from the OQMD')
         data = FormationEnergy.objects.exclude(delta_e=None, composition=None)
+
         if fit:
             data = data.filter(fit=fit)
         else:
             total = True
+
         if stable:
             data = data.exclude(stability__lte=0)
+
         if search:
             data = data.filter(**search)
+
         if space:
             space_qs = Element.objects.exclude(symbol__in=space)
             data = data.filter(composition__element_set__in=space)
@@ -166,8 +168,8 @@ class PhaseData(object):
         values = data.values(*columns)
 
         for row in values:
-            if not row['composition_id']:
-                continue
+            #if not row['composition_id']:
+            #    continue
             if total:
                 energy = row['calculation__energy_pa']
             else:
@@ -351,10 +353,10 @@ class Phase(object):
 
     def __repr__(self):
         if self.description:
-            return '<Phase {name} ({description}): {energy}>'.format(
+            return '<Phase {name} ({description}): {energy:0.3g}>'.format(
                     name=self.name, energy=self.energy, description=self.description)
         else:
-            return '<Phase {name} : {energy}>'.format(
+            return '<Phase {name} : {energy:0.3g}>'.format(
                     name=self.name, energy=self.energy)
 
     def __eq__(self,other):
