@@ -1,5 +1,10 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+# qmpy/analysis/vasp/calculation.py
+
+"""
+The Calculation class is the container for all information about a calculation,
+and is used to read/write/create calculation directories.
+
+"""
 
 import os
 import copy
@@ -9,13 +14,21 @@ import gzip
 import numpy as np
 import numpy.linalg
 import logging
+import re
+import subprocess
 from collections import defaultdict
 from os.path import exists, isfile, isdir
 
+try:
+    from lxml import etree
+except ImportError:
+    import elementtree.ElementTree as etree
+
 from django.db import models
 from django.db import transaction
 
 import qmpy
+import qmpy.materials.composition as comp
 import qmpy.materials.structure as strx
 import qmpy.io.poscar as poscar
 import potential as pot
@@ -23,6 +36,7 @@ import qmpy.materials.formation_energy as fe
 import qmpy.utils as utils
 import qmpy.db.custom as cdb
 import qmpy.analysis.thermodynamics as thermo
+import qmpy.analysis.griddata as grid
 import dos
 from qmpy.data import chem_pots
 from qmpy.materials.atom import Atom, Site
@@ -33,6 +47,9 @@ from qmpy.db.custom import DictField, NumpyArrayField
 from qmpy.configuration.vasp_settings import *
 
 logger = logging.getLogger(__name__)
+#logger.setLevel(logging.DEBUG)
+
+re_iter = re.compile('([0-9]+)\( *([0-9]+)\)')
 
 def value_formatter(value):
     if isinstance(value, list):
@@ -457,10 +474,53 @@ class Calculation(models.Model):
     def POSCAR(self, poscar):
         self.input = poscar.read(poscar)
 
-    #= Results Reading =#
-
+    xmlroot = None
     def read_vasprun_xml(self):
+        tree = etree.parse(gzip.open(self.path+'/vasprun.xml.gz','rb'))
+        self.xmlroot = tree.getroot()
+
+        # read settings
+        settings = {}
+        for s in self.xmlroot.findall('parameters/separator/*'):
+            t = s.get('type', 'float')
+            if not s.text:
+                continue
+            if s.tag == 'i':
+                if t == 'int':
+                    settings[s.get('name').lower()] = int(s.text.strip())
+                elif t == 'float':
+                    settings[s.get('name').lower()] = float(s.text.strip())
+                elif t == 'string':
+                    settings[s.get('name').lower()] = s.text.strip()
+            elif s.tag == 'v':
+                settings[s.get('name').lower()] = map(float, s.text.split())
+        self.settings = settings
+
+        # read other things
+        lattices = []
+        for b in self.xmlroot.findall("structure/crystal/*[@name='basis']"):
+            cell = []
+            for v in b:
+                cell.append(map(float, v.strip().split()))
+            lattices.append(np.vstack(cell))
+
+        # coords
+        positions = []
+        for c in self.xmlroot.findall("structure/varray[@name='positions']"):
+            coords = []
+            for v in c:
+                coords.append(map(float, v.strip().split()))
+            positions.append(np.vstack(coords))
+
         raise NotImplementedError
+        # energies
+        energies = []
+
+        # forces
+        forces = []
+
+        # stresses
+        stresses = []
 
     def get_outcar(self):
         """
@@ -789,6 +849,11 @@ class Calculation(models.Model):
         v_fin = None
         for line in self.outcar[::-1]:
             if 'Iteration' in line:
+                ionic, electronic = map(int, re_iter.findall(line)[0])
+                if self.settings.get('nelm', 60) == electronic:
+                    sc_converged = False
+                if self.settings.get('nsw', 0) == ionic:
+                    forces_converged = False
                 break
             if 'EDIFF is reached' in line:
                 sc_converged = True
@@ -830,6 +895,8 @@ class Calculation(models.Model):
                 settings['istart'] = int(line.split()[2])
             elif 'ISPIN' in line:
                 settings['ispin'] = int(line.split()[2])
+            elif 'ICHARG' in line:
+                settings['icharg'] = int(line.split()[2])
 
             # electronic relaxation 1
             elif 'NELM' in line:
@@ -1007,6 +1074,48 @@ class Calculation(models.Model):
         self.read_outcar_results()
         self.read_n_ionic()
 
+    def read_chgcar(self, filename='CHGCAR.gz', filetype='CHGCAR'):
+        """
+        Reads a VASP CHGCAR or ELFCAR and returns a GridData instance.
+
+        """
+
+        # Determine the number of data columns
+        if 'CHGCAR' in filename:
+            width = 5
+        elif 'ELFCAR' in filename:
+            width = 10
+        else:
+            width = 5
+        
+        if not os.path.exists(self.path+'/'+filename):
+            raise VaspError("%s does not exist at %s", filetype, filename)
+
+        if '.gz' in filename:
+            f = gzip.open('%s/%s' % (self.path,filename),'rb')
+        else:
+            f = open('%s/%s' % (self.path,filename),'r')
+
+        d = f.readlines() 
+        lattice = np.array([map(float, r.split()) for r in d[2:5]])
+        stoich = np.array(d[6].split(),int)
+        count = sum(stoich)
+        meshsize = np.array(d[9+int(count)].split(),int)
+        mesh_spacing = 1./meshsize
+        top = 10+int(count)
+        length = int(np.floor(np.product(meshsize)/width))
+        list = np.array(map(lambda d:
+            np.array(d.strip().split(), float), d[top:top+length]))
+        if np.product(meshsize) % width != 0:
+            trail = d[top+length].rstrip().split()
+            rem = np.product(meshsize) % width
+            list = np.append(list, np.array(trail[0:rem],float))
+
+        new_list = np.reshape(list, meshsize[::-1])
+        self.xdens = grid.GridData(new_list.swapaxes(0,2),
+                lattice=lattice)
+        return self.xdens
+
     def read_doscar(self):
         if os.path.getsize(self.path+'/DOSCAR') < 300:
             return
@@ -1335,6 +1444,77 @@ class Calculation(models.Model):
         if any(self.input.magmoms):
             self.ispin = 2
 
+    def set_wavecar(self, source):
+        """
+        Copy the WAVECAR specified by `source` to this calculation.
+
+        Arguments:
+            source: can be another :mod:`~qmpy.Calculation` instance or a
+            string containing a path to a WAVECAR. If it is a path, it should 
+            be a absolute, i.e. begin with "/", and can either end with the 
+            WAVECAR or simply point to the path that contains it. For
+            example, if you want to take the WAVECAR from a previous 
+            calculation you can do any of::
+            
+            >>> c1 # old calculation
+            >>> c2 # new calculation
+            >>> c2.set_wavecar(c1)
+            >>> c2.set_wavecar(c1.path)
+            >>> c2.set_wavecar(c1.path+'/WAVECAR')
+
+        """
+        if isinstance(source, Calculation):
+            source = calculation.path
+
+        source = os.path.abspath(source)
+        if not os.path.exists(source):
+            raise VaspError('WAVECAR does not exist at %s', source) 
+
+        if not 'WAVECAR' in source:
+            files = os.listdir(source)
+            for f in files:
+                if 'WAVECAR' in f:
+                    new_path = '%s/%s' % (source, f)
+                    self.set_wavecar(new_path)
+        else:
+            subprocess.check_call(['cp', source, self.path])
+
+    def set_chgcar(self, source):
+        """
+        Copy the CHGCAR specified by `source` to this calculation.
+
+        Arguments:
+            source: can be another :mod:`~qmpy.Calculation` instance or a
+            string containing a path to a CHGCAR. If it is a path, it should 
+            be a absolute, i.e. begin with "/", and can either end with the 
+            CHGCAR or simply point to the path that contains it. For
+            example, if you want to take the CHGCAR from a previous 
+            calculation you can do any of::
+            
+            >>> c1 # old calculation
+            >>> c2 # new calculation
+            >>> c2.set_chgcar(c1)
+            >>> c2.set_chgcar(c1.path)
+            >>> c2.set_chgcar(c1.path+'/CHGCAR')
+
+        """
+        if isinstance(source, Calculation):
+            source = source.path
+
+        source = os.path.abspath(source)
+        if not os.path.exists(source):
+            raise VaspError('CHGCAR does not exist at %s', source) 
+
+        if not 'CHGCAR' in source:
+            files = os.listdir(source)
+            for f in files:
+                if 'CHGCAR' in f:
+                    new_path = '%s/%s' % (source, f)
+                    self.set_chgcar(new_path)
+        else:
+            logger.debug('Copying: %s to %s', source, self.path)
+            subprocess.check_call(['cp', source, self.path])
+
     @property
     def volume(self):
         if self.output:
@@ -1352,7 +1532,7 @@ class Calculation(models.Model):
         if not self.converged:
             return None
         if len(self.input.comp) == 1:
-            e = self.input.composition.total_energy
+            e = comp.Composition.get(self.input.comp).total_energy
             formation = fe.FormationEnergy.get(self, fit=reference)
             formation.delta_e = self.energy_pa - e
             formation.composition = self.input.composition
@@ -1375,7 +1555,9 @@ class Calculation(models.Model):
 
     @staticmethod
     def setup(input=None, configuration='standard', path=None, entry=None,
-            hubbard='wang', potentials='vasp_rec', settings={}, **kwargs):
+            hubbard='wang', potentials='vasp_rec', settings={}, 
+            chgcar=None, wavecar=None,
+            **kwargs):
         
         if path is None:
             path = os.path.abspath(entry.path)
@@ -1388,8 +1570,12 @@ class Calculation(models.Model):
             if not os.path.exists(path):
                 os.mkdir(path)
             calc = Calculation()
-            calc.configuration = configuration
             calc.path = path
+            calc.configuration = configuration
+            if chgcar:
+                calc.set_chgcar(chgcar)
+            if wavecar:
+                calc.set_wavecar(wavecar)
             calc.input = input
             calc.kwargs = kwargs
             if entry:
@@ -1444,11 +1630,12 @@ class Calculation(models.Model):
         fixed_calc = calc.address_errors()
         if fixed_calc.errors:
             # unable to fix errors
-            return fixed_calc
+            raise VaspError("Unable to fix errors: %s", fixed_calc.errors)
 
         fixed_calc.set_magmoms(calc.settings.get('magnetism', 'ferro'))
         fixed_calc.clear_results()
         calc.backup()
         fixed_calc.clear_outputs()
+        fixed_calc.set_chgcar(calc)
         fixed_calc.write()
         return fixed_calc
