@@ -112,7 +112,6 @@ class TaskManager(daemon.Daemon):
             self.handle_task(task, host)
 
     def handle_task(self, task, host):
-        print task
         os.umask(022)
         t0 = time.time()
         tlogger.debug('Attempting task: %s belonging to entry %s', 
@@ -126,11 +125,11 @@ class TaskManager(daemon.Daemon):
             task.save()
             tlogger.warn('VASP error')
             return
-        #except Exception, err:
-        #    task.state = -2
-        #    task.save()
-        #    tlogger.warn('Unknown error reading VASP outputs: %s', err)
-        #    return
+        except Exception, err:
+            task.state = -2
+            task.save()
+            tlogger.warn('Unknown error reading VASP outputs: %s', err)
+            return
 
         if not jobs:
             task.complete()
