@@ -6,24 +6,26 @@ import os
 
 import qmpy.utils as utils
 from qmpy.analysis.vasp import *
+from qmpy.analysis.thermodynamics.space import PhaseSpace
 
 logger = logging.getLogger(__name__)
 
-def initialize(entry,*args, **kwargs):
+def initialize(entry, **kwargs):
     entry.input.set_magnetism('ferro')
     calc = Calculation.setup(entry.input, entry=entry,
             configuration='initialize', path=entry.path+'/initialize', 
-            kwargs=kwargs)
+            **kwargs)
     if not calc.converged:
         calc.write()
         return calc
-
     if calc.converged:
+        if calc.magmom > 0.1:
+            entry.keywords.append('magnetic')
         entry.calculations['initialize'] = calc
-        calc.output = calc.input
+        entry.structures['initialize'] = calc.output
     return calc
 
-def coarse_relax(entry, *args, **kwargs):
+def coarse_relax(entry, **kwargs):
     if entry.calculations.get('coarse_relax', Calculation()).converged:
         return entry.calculations['coarse_relax']
 
@@ -33,6 +35,8 @@ def coarse_relax(entry, *args, **kwargs):
         return calc
 
     inp = entry.structures['input']
+    if not 'magnetic' in entry.keywords:
+        inp.set_magnetism('none')
     calc = Calculation.setup(inp, entry=entry,
         configuration='coarse_relax', path=entry.path+'/coarse_relax',
         **kwargs)
@@ -42,7 +46,7 @@ def coarse_relax(entry, *args, **kwargs):
         entry.calculations['coarse_relax'] = calc
     return calc
 
-def fine_relax(entry, *args, **kwargs):
+def fine_relax(entry, **kwargs):
     if entry.calculations.get('fine_relax', Calculation()).converged:
         return entry.calculations['fine_relax']
 
@@ -51,17 +55,17 @@ def fine_relax(entry, *args, **kwargs):
         calc.write()
         return calc
     inp = entry.structures['coarse_relax']
-    inp.set_magnetism('ferro')
+    if 'magnetic' in entry.keywords:
+        inp.set_magnetism('ferro')
 
     calc = Calculation.setup(inp, entry=entry,
-        configuration='fine_relax', path=entry.path+'/fine_relax',
-        *args, **kwargs)
+        configuration='fine_relax', path=entry.path+'/fine_relax', **kwargs)
     if calc.converged:
         entry.structures['fine_relax'] = calc.output
         entry.calculations['fine_relax'] = calc
     return calc
 
-def standard(entry, *args, **kwargs):
+def standard(entry, **kwargs):
     if entry.calculations.get('standard', Calculation()).converged:
         return entry.calculations['standard']
 
@@ -71,25 +75,27 @@ def standard(entry, *args, **kwargs):
         return calc
 
     inp = entry.structures['fine_relax']
-    inp.set_magnetism('ferro')
+    if 'magnetic' in entry.keywords:
+        inp.set_magnetism('ferro')
 
     calc = Calculation.setup(inp, entry=entry,
         configuration='standard', path=entry.path+'/standard', **kwargs)
     if calc.converged:
-        calc.output = calc.input
-        calc.compute_formation()
+        f = calc.compute_formation()
+        f.entry = entry
+        f.save()
         entry.calculations['standard'] = calc
+        entry.structures['standard'] = calc.output
     return calc
 
-def relaxation(entry, *args, **kwargs):
+def relaxation(entry, **kwargs):
     if entry.calculations.get('relaxation', Calculation()).converged:
         return entry.calculations['relaxation']
 
     input = entry.input
     input.make_primitive()
-    input.set_magnetism('ferro')
 
-    calc = Calculation.setup(input, *args, entry=entry,
+    calc = Calculation.setup(input,  entry=entry,
                                      configuration='relaxation',
                                      path=entry.path+'/relaxation',
                                      **kwargs)
@@ -101,25 +107,28 @@ def relaxation(entry, *args, **kwargs):
         entry.structures['relaxation'] = calc.output
     return calc
 
-def static(entry, *args, **kwargs):
+def static(entry, **kwargs):
     if entry.calculations.get('static', Calculation()).converged:
         return entry.calculations['static']
 
-    calc = relaxation(entry, *args, **kwargs)
+    calc = relaxation(entry, **kwargs)
     if not calc.converged:
         calc.write()
         return calc
 
     input = calc.output
-    input.set_magnetism('ferro')
     calc = Calculation.setup(input, entry=entry,
                                     configuration='static', 
                                     path=entry.path+'/static', 
+                                    chgcar=entry.path+'/relaxation',
                                     **kwargs)
     entry.calculations['static'] = calc
 
     if calc.converged:
-        calc.output = calc.input
-        calc.compute_formation()
+        f = calc.compute_formation()
+        f.save()
+        s = PhaseSpace('-'.join(entry.comp.keys()))
+        s.compute_stabilities()
         entry.calculations['static'] = calc
+        entry.structures['final'] = calc.output
     return calc
