@@ -82,7 +82,7 @@ class Atom(models.Model):
         db_table = 'atoms'
 
     def __str__(self):
-        return '%s @ %3f %3f %3f' % (self.element_id, 
+        return '%s @ %0.3g %0.3g %0.3g' % (self.element_id, 
                 self.x, self.y, self.z)
 
     def __eq__(self, other):
@@ -235,7 +235,7 @@ class Atom(models.Model):
         if not self.structure is None:
             for site in self.structure.sites:
                 if self.is_on(site):
-                    site.add_atom(self)
+                    site.add_atom(self, tol=tol)
                     return site
 
         s = Site()
@@ -292,7 +292,7 @@ class Site(models.Model):
         coord_str = ''
         comp_str = ''
         if not self.coord is None:
-            coord_str = '%0.3f %0.3f %0.3f' % tuple(self.coord)
+            coord_str = '%0.3g %0.3g %0.3g' % tuple(self.coord)
 
         if len(self.atoms) == 1:
             comp_str = self.atoms[0].element_id
@@ -439,7 +439,7 @@ class Site(models.Model):
 
         site.coord = atoms[0].coord
         for a in atoms:
-            site.add_atom(a)
+            site.add_atom(a, tol=tol)
         return site
 
     @staticmethod
@@ -543,7 +543,7 @@ class Site(models.Model):
             comp[a.species] += a.occupancy
         return dict(comp)
 
-    def add_atom(self, atom, tol=1e-3):
+    def add_atom(self, atom, tol=1e-2):
         """
         Adds Atom to `Site.atoms`. 
 
@@ -621,4 +621,4 @@ class Site(models.Model):
 
     @property
     def partial(self):
-        return any([ abs(1 - a.occupancy) > 0.001 for a in self ])
+        return any([ a.occupancy != 1.0 for a in self ])
