@@ -140,12 +140,17 @@ class Composition(models.Model):
     @property
     def entries(self):
         entries = self.entry_set.filter(duplicate_of=None)
+        if not entries.exists():
+            return []
         return sorted(entries, key=lambda x:
                      100 if x.energy is None else x.energy )
 
     @property
     def ground_state(self):
         """Return the most stable entry at the composition."""
+        e = self.entries
+        if not e:
+            return None
         return self.entries[0]
 
     _elements = None
