@@ -94,11 +94,11 @@ class Entry(models.Model):
                 self.reference.save()
                 self.reference = self.reference
         super(Entry, self).save(*args, **kwargs)
-        #for k, v in self.structures.items():
-        #    v.label = k
+        for k, v in self.structures.items():
+            v.label = k
         self.structure_set = self.structures.values()
-        #for k, v in self.calculations.items():
-        #    v.label = k
+        for k, v in self.calculations.items():
+            v.label = k
         self.calculation_set = self.calculations.values()
         if self._elements:
             self.element_set = self.elements
@@ -142,11 +142,6 @@ class Entry(models.Model):
         # Step 2
         entry = Entry(**kwargs)
         structure = io.read(source_file)
-        try:
-            s2 = structure.make_perfect(in_place=False)
-            structure = s2
-        except StructureError:
-            pass
         structure.make_primitive()
         entry.source_file = source_file
         entry.path = os.path.dirname(source_file)
@@ -184,24 +179,6 @@ class Entry(models.Model):
             entry.add_hold('partial occupancy')
 
         return entry
-        # Step 4
-        #except:
-        #    return entry
-
-        c3 = perfect.composition
-        if c3.compare(c1, 5e-2):
-            entry.structures['raw'] = entry.input
-            entry.input = perfect
-            entry.composition = perfect.composition
-            entry.remove_hold('partial occupancy')
-            entry.add_keyword('adjusted occupancy')
-        dup = Entry.get(perfect)
-
-        if not dup is None:
-            entry.add_hold('duplicate')
-            entry.duplicate_of = dup
-
-        return entry
 
     @staticmethod
     def get(structure):
@@ -267,12 +244,10 @@ class Entry(models.Model):
             if self.id is None:
                 self._structures = {}
             else:
-                structs = self.structure_set.exclude(label='')
-                labels = [ s.label for s in structs ]
-                if len(set(labels)) < len(labels):
-                    logger.warn('Warning! Some objects have repeat labels')
-                    labels = [ '%s_%s' % (s.label, s.pk) for s in structs ]
-                self._structures = dict((k, s) for k,s in zip(labels, structs))
+                structs = {}
+                for s in self.structure_set.exclude(label=''):
+                    structs[s.label] = s
+                self._structures = structs
         return self._structures
 
     @structures.setter
@@ -296,12 +271,10 @@ class Entry(models.Model):
             if self.id is None:
                 self._calculations = {}
             else:
-                calcs = self.calculation_set.exclude(label='')
-                labels = [ c.label for c in calcs ]
-                if len(set(labels)) < len(labels):
-                    logger.warn('Warning! Some objects have repeat labels')
-                    labels = [ '%s_%s' % (c.label, c.pk) for c in calcs ]
-                self._calculations = dict((k, c) for k,c in zip(labels, calcs))
+                calcs = {}
+                for c in self.calculation_set.exclude(label=''):
+                    calcs[c.label] = c
+                self._calculations = calcs
         return self._calculations
 
     @calculations.setter
@@ -394,11 +367,11 @@ class Entry(models.Model):
     def proto_label(self):
         #if not self.prototype is None:
         #    return self.prototype.name
-        similar = self.input.similar
         protos = []
-        for s in similar:
-            if not s.entry.prototype is None:
-                protos.append(s.entry.prototype.name)
+        for e in self.duplicates.all():
+            if not e.prototype is None:
+                protos.append(e.prototype.name)
+                
         protos = list(set(protos))
         if len(protos) == 1:
             return protos[0]
@@ -428,14 +401,16 @@ class Entry(models.Model):
         es = []
         if 'static' in self.calculations:
             if self.calculations['static'].converged:
-                es.append(self.calculations['static'].energy_pa)
+                return self.calculations['static'].energy_pa
+                #es.append(self.calculations['static'].energy_pa)
         if 'standard' in self.calculations:
             if self.calculations['standard'].converged:
-                es.append(self.calculations['standard'].energy_pa)
+                return self.calculations['standard'].energy_pa
+                #es.append(self.calculations['standard'].energy_pa)
         if not es:
             return None
-        else:
-            return min(es)
+        #else:
+        #    return min(es)
 
     _energy = None
     @property
@@ -445,11 +420,15 @@ class Entry(models.Model):
         final relaxed structure. Otherwise, returns None.
 
         """
-        if not self._energy is None:
-            return self._energy
-        forms = self.formationenergy_set.all()
-        if forms.exists():
-            self._energy = min(forms.values_list('delta_e', flat=True))
+        if self._energy is None:
+            if 'static' in self.calculations:
+                if self.calculations['static'].converged:
+                    de = self.calculations['static'].compute_formation()
+                    self._energy = de.delta_e
+            elif 'standard' in self.calculations:
+                if self.calculations['standard'].converged:
+                    de = self.calculations['standard'].compute_formation()
+                    self._energy = de.delta_e
         return self._energy
 
     _history = None
@@ -486,9 +465,7 @@ class Entry(models.Model):
 
     @property
     def spacegroup(self):
-        for key in ['relaxed', 'fine_relaxed', 'input']:
-            if key in self.structures:
-                return self.structures[key].spacegroup
+        return self.structure.spacegroup
 
     @property
     def mass(self):
@@ -593,9 +570,7 @@ class Entry(models.Model):
             job.collect()
             job.delete()
 
-        print self.job_set.count()
         self.job_set.all().delete()
-        print self.job_set.count()
 
         for dir in os.listdir(self.path):
             if os.path.isdir(self.path+'/'+dir):
