@@ -155,6 +155,16 @@ class Structure(models.Model, object):
     def __str__(self):
         return format_comp(reduce_comp(self.comp))
 
+    def printf(self):
+        res = format_comp(reduce_comp(self.comp)) + '\n'
+        res += self.lat_param_string()
+        for i, s in enumerate(self.sites):
+            res += '\n - %s' % s
+            if i == 6:
+                res += '\n ... \n %d more atoms.' % (len(self)-6)
+                break
+        return res
+
     def __getitem__(self, i):
         return self.atoms[i]
 
@@ -242,10 +252,10 @@ class Structure(models.Model, object):
 
     @atoms.setter
     def atoms(self, atoms):
-        self.natoms = len(atoms)
         self._atoms = []
         for a in atoms:
             self.add_atom(a)
+        self.natoms = len(self._atoms)
         self.ntypes = len(self.comp)
 
     _sites = None
@@ -257,6 +267,7 @@ class Structure(models.Model, object):
         if self._sites is None:
             self.atoms
             self.get_sites()
+            self.nsites = len(self.sites)
         return self._sites
 
     @sites.setter
@@ -279,6 +290,7 @@ class Structure(models.Model, object):
             s.comp = parse_comp(v)
             atoms += s.atoms
         self._atoms = atoms
+        self.natoms = len(atoms)
 
     @property
     def elements(self):
@@ -342,6 +354,55 @@ class Structure(models.Model, object):
         self.cell = latparams_to_basis(lat_params)
         self._lat_params = lat_params
 
+    def lat_param_string(self, format='screen'):
+        """
+        Generates a human friendly representation of the lattice parameters of
+        a structure.
+
+        Keyword Arguments:
+            format: ('screen'|'html'|'mathtype')
+
+        """
+        formats = {
+                'html':{
+                    'keys':['&alpha;', '&beta;', '&gamma'],
+                    'newline':'<br>'},
+                'mathtype':{
+                    'keys':[r'\alpha', r'\beta', r'\gamma'],
+                    'newline':'\n'},
+                'screen':{
+                    'keys':[r'alpha', r'beta', r'gamma'],
+                    'newline':'\n'}
+                }
+
+        f = formats[format]
+
+        lp = self.lat_param_dict
+        if abs(lp['a'] - lp['b']) < 1e-4:
+            if abs(lp['a'] - lp['c']) < 1e-4:
+                lpstr = 'a = b = c = %0.3g' % lp['a']
+            else:
+                lpstr = 'a = b = %0.3g, c = %0.3g' % ( lp['a'], lp['c'])
+        else:
+            lpstr = 'a = %0.3g, b = %0.3g, c = %0.3g' % (lp['a'], lp['b'], lp['c'])
+
+        lpstr += f['newline']
+
+        if abs(lp['alpha'] - lp['beta']) < 1e-2:
+            if abs(lp['alpha'] - lp['gamma']) < 1e-2:
+                lpstr += '%s = %s = %s = %0.3g' % (
+                    f['keys'][0], f['keys'][1], f['keys'][2], lp['alpha'])
+            else:
+                lpstr += '%s = %s = %0.3g, %s = %0.3g' % (
+                    f['keys'][0], f['keys'][1], lp['alpha'], 
+                    f['keys'][2], lp['gamma'])
+        else:
+            lpstr += '&alpha; = %0.3g, &beta; = %0.3g, &gamma; = %0.3g' % (
+                f['keys'][0], lp['alpha'],  
+                f['keys'][1], lp['beta'],
+                f['keys'][2], lp['gamma'])
+        return lpstr
+
     lp = lat_params
 
     @property
@@ -545,8 +606,11 @@ class Structure(models.Model, object):
         """
 
         # 1
+        if len(self) > 80 or len(other) > 80:
+            return False
         me = self.copy()
         you = other.copy()
+
         if not set(me.elements) == set(you.elements):
             logger.debug("Structure comparison: element mismatch")
             return False
@@ -646,7 +710,7 @@ class Structure(models.Model, object):
         return False
 
     def contains(self, atom, tol=0.01):
-        atom.dist = la.norm(np.dot(atom.coord, self.cell))
+        atom.dist = la.norm(np.dot(atom.coord-np.round(atom.coord), self.cell))
         for atom2 in self.atoms:
             if not atom2.element_id == atom.element_id:
                 continue
@@ -719,6 +783,7 @@ class Structure(models.Model, object):
         if limit:
             if dist > limit:
                 return None
+
         return dist
 
     def add_atom(self, atom, tol=0.01):
@@ -731,7 +796,7 @@ class Structure(models.Model, object):
         self.atoms.append(atom)
         for site in self.sites:
             if atom.is_on(site, tol=tol):
-                site.add_atom(atom)
+                site.add_atom(atom, tol=tol)
                 break
         else:
             site = atom.get_site()
@@ -939,8 +1004,10 @@ class Structure(models.Model, object):
         """
         new = Structure()
         new.cell = self.cell
+        new.sites = []
         new.atoms = [ atom.copy() for atom in self.atoms ]
         new.entry = self.entry
+        new.composition = self.composition
         return new
 
     @property
@@ -953,12 +1020,12 @@ class Structure(models.Model, object):
     def set_natoms(self, n):
         """Set self.atoms to n blank Atoms."""
         self.atoms = [ Atom() for i in range(n) ]
-        self.sites = []
+        self._sites = None
 
     def set_nsites(self, n):
         """Sets self.sites to n blank Sites."""
         self.sites = [ Site() for i in range(n) ]
-        self.atoms = []
+        self._atoms = None
 
     def make_conventional(self, in_place=True, tol=1e-5):
         """Uses spglib to convert to the conventional cell.
@@ -1023,13 +1090,15 @@ class Structure(models.Model, object):
 
         """
         if not any([ a.occupancy < 1 for a in self.atoms ]):
-            self.sites = [ a.get_site() for a in self.atoms ]
+            self._sites = [ a.get_site() for a in self.atoms ]
             return self._sites
 
         _sites = []
         for atom in self.atoms:
             site = atom.get_site()
             site.structure = self
+            if not any([ site is site2 for site2 in _sites ]):
+                _sites.append(site)
         self._sites = _sites
         return self.sites
 
@@ -1284,11 +1353,28 @@ class Structure(models.Model, object):
             cv = np.array(map(float, np.dot(self.inv.T, cv)))
 
         coords = self.coords + cv
-        coords = coords % 1.0
-        self.coords = coords % 1.0
+        self.coords = wrap(coords)
         return self
 
-    def find_lattice_points(self, transform, tol=1e-6):
+    def find_lattice_points_within_distance(self, distance, tol=1e-6):
+        """
+        Find the lattice points contained within radius `distance` from the
+        origin.
+
+        """
+        limits = [ int(np.ceil(distance/self.lp[i])) for i in range(3) ]
+        ranges = [ range(-l, l+1) for l in limits ]
+        d2 = distance**2
+        lattice_points = []
+        for i, j, k in itertools.product(*ranges):
+            point = np.dot([i,j,k], self.cell)
+            if any([ abs(p) > distance for p in point ]):
+                continue
+            if np.dot(point, point) < d2:
+                lattice_points.append([i,j,k])
+        return np.vstack(lattice_points)
+
+    def find_lattice_points_by_transform(self, transform, tol=1e-6):
         """
         Find the lattice points contained within the transformation.
 
@@ -1398,7 +1484,7 @@ class Structure(models.Model, object):
         n_cells = abs(roundclose(la.det(transform)))
 
         atoms = []
-        for point in self.find_lattice_points(transform):
+        for point in self.find_lattice_points_by_transform(transform):
             for atom in self.atoms:
                 new = atom.copy()
                 coord = point + atom.coord
@@ -1647,7 +1733,28 @@ class Structure(models.Model, object):
         self.pdf.get_pair_distances()
         return self.pdf
 
-    def make_perfect(self, in_place=True):
+    _cart_rots = None
+    def get_cartesian_rotations(self):
+        if self._cart_rots is None:
+            self.symmetrize()
+            p = self.cell.T
+            q = la.inv(self.cell.T)
+            cr = []
+            for rot in self.rotations:
+                c1 = p.dot(rot).dot(q)
+                if any([ np.allclose(c1, c2) for c2 in cr ]):
+                    continue
+                cr.append(c1)
+            self._cart_rots = cr
+        return self._cart_rots
+
+    @property
+    def is_perfect(self):
+        if any([s.partial for s in self.sites ]):
+            return False
+        return True
+
+    def make_perfect(self, in_place=True, tol=1e-1):
         """
         Constructs options for a 'perfect' lattice from the structure.
 
@@ -1660,18 +1767,20 @@ class Structure(models.Model, object):
             in_place: If False returns a new :mod:`~qmpy.Structure`, otherwise
             returns None
 
+            tol: maximum defect concentration.
+
         Examples::
             
             >>> s = io.read(INSTALL_PATH+'/io/files/partial_vac.cif')
             >>> s
             <Structure: Mn3.356Si4O16>
-            >>> s.get_perfect_structure()
+            >>> s.make_perfect()
             >>> s
             <Structure: MnSiO4>
             >>> s = io.read(INSTALL_PATH+'/io/files/partial_mix.cif')
             >>> s
             <Structure: Mn4.264Co3.736Si4O16>
-            >>> s2 = s.get_perfect_structure(in_place=False)
+            >>> s2 = s.make_perfect(in_place=False)
             >>> s2
             <Structure: MnCoSiO4>
 
@@ -1680,37 +1789,57 @@ class Structure(models.Model, object):
             new = self.copy()
             return new.make_perfect(True)
 
-        pdf = self.get_pdf(limit=None)
-        for k,v in pdf.distances.items():
-            d = min(v)
+        init_atoms = [ a.copy() for a in self.atoms ]
+        init_comp = dict(self.comp)
+        n = sum(init_comp.values())
+        atom_tol = tol*n
+        hopeless = False
+
+        for s1, s2 in itertools.combinations(self.sites, r=2):
+            d = self.get_distance(s1, s2, limit=1, wrap_self=True) 
+            if d is None:
+                continue
             if d < 0.8:
-                raise StructureError("Atoms too close together")
+                return self
 
-        inds = []
-        for site in self.sites:
-            if len(site) == 1 and site.occupancy != 1.0:
-                inds.append(site.atoms[0].index)
-            elif len(site) > 1:
-                dom = None
-                occs = [ a.occupancy for a in site ]
-                comps = [ self.unit_comp[a.element_id] for a in site ]
-                if max(occs) > min(occs):
-                    dom = sorted(site.atoms, key=lambda x: x.occupancy)[0]
-                elif max(comps) > min(comps):
-                    dom = sorted(site.atoms, key=lambda x:
-                                        self.comp[x.element_id])[0]
-                else:
-                    raise StructureError(
-                            "Cannot assign species to equal mixture")
-
-                inds.append(dom.index)
-            else:
-                inds.append(site.atoms[0].index)
-
-        self.atoms = np.array(self.atoms)[inds]
+        atoms = []
         for atom in self.atoms:
-            atom.occupancy = 1.0
-        self.composition = Composition.get(self.comp)
+            if abs(1-atom.occupancy) < 0.5:
+                # first, fill any nearly full sites
+                new = atom.copy()
+                new.occupancy = 1.0
+                new.site = None
+                atoms.append(new)
+            elif abs(atom.occupancy) < 0.5:
+                # then, empty any nearly empty sites
+                continue
+            elif atom.occupancy >= 2:
+                raise StructureError('Site occupied by a molecule')
+            else:
+                continue
+
+        self._sites = []
+        self.atoms = atoms
+        self.get_sites()
+
+        if self.is_perfect:
+            # total
+            if abs(1 - sum(self.comp.values())/n) > tol:
+                hopeless = True
+
+            for k in init_comp.keys():
+                d = init_comp[k] - self.comp.get(k,0.0)
+                if abs(d) > n*tol:
+                    hopeless = True
+                    break
+            self.composition = Composition.get(self.comp)
+            
+            if not hopeless:
+                return self
+
+        self._sites = []
+        self.atoms = init_atoms 
+        self.get_sites()
         return self
 
 class Prototype(models.Model):
@@ -1751,3 +1880,11 @@ class Prototype(models.Model):
         if new:
             obj.save()
         return obj
+
+    def __str__(self):
+        if not self.structure is None:
+            sname = self.structure.name
+            return '%s - %s' % (self.name, self.structure.name)
+        else:
+            return self.name
+
