@@ -1,11 +1,12 @@
 #!/usr/bin/env python
 
 #= Represent materials, and their attributes =#
+from materials.atom import *
 from materials.entry import *
 from materials.structure import *
 from materials.composition import *
 from materials.element import *
-from materials.xrd import *
+from materials.formation_energy import *
 
 #= DFT Calculations and computed properties =#
 from analysis.vasp.calculation import *
@@ -23,7 +24,7 @@ from analysis.symmetry import *
 
 #= Known experimental data, and meta data =#
 from data.meta_data import *
-from data.experiment import *
+from data.reference import *
 
 #= Custom database models and fields =#
 from qmpy.db.custom import *
