@@ -274,10 +274,9 @@ def reduce_comp(values, method='auto'):
     if len(ints) == len(values) - 1:
         return make_return(second)
 
-    third = reduce_by_any_means(values)
-    if all( v < 1000 for v in third):
-        return make_return(third)
-
+    #third = reduce_by_any_means(values)
+    #if all( v < 1000 for v in third):
+    #    return make_return(third)
     return make_return(second)
 
 def normalize_comp(values):
